'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

exports['default'] = graphql;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _parse = require('./parse');

var _parse2 = _interopRequireDefault(_parse);

var _traverse = require('./traverse');

var _traverse2 = _interopRequireDefault(_traverse);

function join(parts) {
  var result = parts[0] || '';
  for (var i = 1; i < parts.length; i++) {
    result += '&' + (i - 1) + parts[i];
  }

  return result;
}

function graphql(strings) {
  for (var _len = arguments.length, values = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    values[_key - 1] = arguments[_key];
  }

  var source = join(strings);
  var ast = (0, _parse2['default'])(source);

  return function (params) {
    return (0, _traverse2['default'])(ast, new TaggedTemplateVisitor(params, values));
  };
}

var rootSymbol = '@@root';
var nameSymbol = '@@name';

var TaggedTemplateVisitor = (function () {
  function TaggedTemplateVisitor(params, quasis) {
    _classCallCheck(this, TaggedTemplateVisitor);

    this.params = params;
    this.quasis = quasis;
  }

  _createClass(TaggedTemplateVisitor, [{
    key: 'transformFields',
    value: function transformFields(fields) {
      var obj = {};

      for (var i = 0; i < fields.length; i++) {
        var f = fields[i];
        if (f.hasOwnProperty(rootSymbol)) {
          for (var field in f) {
            obj[field] = f[field];
          }
        } else {
          obj[f[nameSymbol]] = f;
        }
      }

      return obj;
    }
  }, {
    key: 'Query',
    value: function Query(node) {
      var query = this.transformFields(node.fields);
      Object.defineProperty(query, rootSymbol, { value: true });

      return query;
    }
  }, {
    key: 'Field',
    value: function Field(node) {
      var field = {};
      Object.defineProperty(field, nameSymbol, { value: node.name });

      if (node.alias) {
        field.alias = node.alias;
      }

      if (node.params.length > 0) {
        field.params = {};

        for (var i = 0; i < node.params.length; i++) {
          var arg = node.params[i];
          field.params[arg.name] = arg.value;
        }
      }

      if (node.fields.length > 0) {
        field.fields = this.transformFields(node.fields);
      }

      return field;
    }
  }, {
    key: 'Literal',
    value: function Literal(node) {
      return node.value;
    }
  }, {
    key: 'Reference',
    value: function Reference(node) {
      return this.quasis[node.name];
    }
  }, {
    key: 'Variable',
    value: function Variable(node) {
      return this.params[node.name];
    }
  }]);

  return TaggedTemplateVisitor;
})();

module.exports = exports['default'];