'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) subClass.__proto__ = superClass; }

var _tokenizer = require('./tokenizer');

var _tokenizer2 = _interopRequireDefault(_tokenizer);

var Parser = (function (_Tokenizer) {
  function Parser() {
    _classCallCheck(this, Parser);

    if (_Tokenizer != null) {
      _Tokenizer.apply(this, arguments);
    }
  }

  _inherits(Parser, _Tokenizer);

  _createClass(Parser, [{
    key: 'match',
    value: function match(type) {
      return this.lookahead.type === type;
    }
  }, {
    key: 'eat',
    value: function eat(type) {
      if (this.match(type)) {
        return this.lex();
      }

      return null;
    }
  }, {
    key: 'expect',
    value: function expect(type) {
      if (this.match(type)) {
        return this.lex();
      }

      throw this.createUnexpected(this.lookahead);
    }
  }, {
    key: 'parseQuery',
    value: function parseQuery() {
      return { type: 'Query', fields: this.parseFieldList() };
    }
  }, {
    key: 'parseIdentifier',
    value: function parseIdentifier() {
      return this.expect(_tokenizer.TokenType.IDENTIFIER).value;
    }
  }, {
    key: 'parseFieldList',
    value: function parseFieldList() {
      this.expect(_tokenizer.TokenType.LBRACE);

      var fields = [];
      var first = true;

      while (!this.match(_tokenizer.TokenType.RBRACE) && !this.end()) {
        if (first) {
          first = false;
        } else {
          this.expect(_tokenizer.TokenType.COMMA);
        }

        if (this.match(_tokenizer.TokenType.AMP)) {
          fields.push(this.parseReference());
        } else {
          fields.push(this.parseField());
        }
      }

      this.expect(_tokenizer.TokenType.RBRACE);
      return fields;
    }
  }, {
    key: 'parseField',
    value: function parseField() {
      var name = this.parseIdentifier();
      var params = this.match(_tokenizer.TokenType.LPAREN) ? this.parseArgumentList() : [];
      var alias = this.eat(_tokenizer.TokenType.AS) ? this.parseIdentifier() : null;
      var fields = this.match(_tokenizer.TokenType.LBRACE) ? this.parseFieldList() : [];

      return { type: 'Field', name: name, alias: alias, params: params, fields: fields };
    }
  }, {
    key: 'parseArgumentList',
    value: function parseArgumentList() {
      var args = [];
      var first = true;

      this.expect(_tokenizer.TokenType.LPAREN);

      while (!this.match(_tokenizer.TokenType.RPAREN) && !this.end()) {
        if (first) {
          first = false;
        } else {
          this.expect(_tokenizer.TokenType.COMMA);
        }

        args.push(this.parseArgument());
      }

      this.expect(_tokenizer.TokenType.RPAREN);
      return args;
    }
  }, {
    key: 'parseArgument',
    value: function parseArgument() {
      var name = this.parseIdentifier();
      this.expect(_tokenizer.TokenType.COLON);
      var value = this.parseValue();

      return { type: 'Argument', name: name, value: value };
    }
  }, {
    key: 'parseValue',
    value: function parseValue() {
      switch (this.lookahead.type) {
        case _tokenizer.TokenType.AMP:
          return this.parseReference();

        case _tokenizer.TokenType.LT:
          return this.parseVariable();

        case _tokenizer.TokenType.NUMBER:
        case _tokenizer.TokenType.STRING:
          return { type: 'Literal', value: this.lex().value };

        case _tokenizer.TokenType.NULL:
        case _tokenizer.TokenType.TRUE:
        case _tokenizer.TokenType.FALSE:
          return { type: 'Literal', value: JSON.parse(this.lex().value) };
      }

      throw this.createUnexpected(this.lookahead);
    }
  }, {
    key: 'parseReference',
    value: function parseReference() {
      this.expect(_tokenizer.TokenType.AMP);

      if (this.match(_tokenizer.TokenType.NUMBER) || this.match(_tokenizer.TokenType.IDENTIFIER)) {
        return { type: 'Reference', name: this.lex().value };
      }

      throw this.createUnexpected(this.lookahead);
    }
  }, {
    key: 'parseVariable',
    value: function parseVariable() {
      this.expect(_tokenizer.TokenType.LT);
      var name = this.expect(_tokenizer.TokenType.IDENTIFIER).value;
      this.expect(_tokenizer.TokenType.GT);

      return { type: 'Variable', name: name };
    }
  }]);

  return Parser;
})(_tokenizer2['default']);

exports['default'] = Parser;
module.exports = exports['default'];